package com.keao.edu.auth.service.impl;

import java.util.Date;
import java.util.List;

import com.keao.edu.auth.dal.dao.SysUserRoleDao;
import com.keao.edu.common.exception.BizException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.keao.edu.auth.api.entity.SysRole;
import com.keao.edu.auth.dal.dao.SysRoleDao;
import com.keao.edu.auth.dal.dao.SysRoleMenuDao;
import com.keao.edu.auth.service.SysRoleService;
import com.keao.edu.common.dal.BaseDAO;
import com.keao.edu.common.service.impl.BaseServiceImpl;

@Service
public class SysRoleServiceImpl extends BaseServiceImpl<Integer, SysRole>  implements SysRoleService {
	
	@Autowired
	private SysRoleDao sysRoleDao;
	@Autowired
	private SysUserRoleDao sysUserRoleDao;
	@Autowired
	private SysRoleMenuDao sysRoleMenuDao;

	@Override
	public BaseDAO<Integer, SysRole> getDAO() {
		return sysRoleDao;
	}

	@Override
	public List<SysRole> findRoleByUserId(Integer userId) {
		return sysRoleDao.findRoleByUserId(userId);
	}

	@Override
	public SysRole findRoleByCode(String code){
		return sysRoleDao.findRoleByCode(code);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void addRole(SysRole sysRole) {
		SysRole sysRole1 = sysRoleDao.findROleByNameAndTenantId(sysRole.getRoleName(),sysRole.getTenantId());
		if(sysRole1 != null){
			throw new BizException("创建失败：角色已存在");
		}
		sysRoleDao.insert(sysRole);
		batchSave(sysRole);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateRole(SysRole sysRole) {
		sysRole.setUpdateTime(new Date());
		sysRoleDao.update(sysRole);
		sysRoleMenuDao.deleteAllMenu(sysRole.getId());
		batchSave(sysRole);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void delRole(Integer id) {
		//是否有关联的用户
		int num = sysUserRoleDao.countRoleUser(id);
		if(num > 0){
			throw new BizException("删除失败：当前角色有关联的用户");
		}
		sysRoleDao.delete(id);
		sysRoleMenuDao.deleteAllMenu(id);
	}

	@Override
	public SysRole getRole(Integer id) {
		SysRole sysRole = sysRoleDao.get(id);
		sysRole.setMenuIds(sysRoleMenuDao.queryMenuIdByRoleId(id));
		return sysRole;
	}

	private void batchSave(SysRole sysRole){
		List<Integer> menuIds = sysRole.getMenuIds();
		if(menuIds != null && menuIds.size() > 0){
//			SysRoleMenu sysRoleMenu = new SysRoleMenu(sysRole.getId(),null);
			sysRoleMenuDao.batchAdd(sysRole.getId(),menuIds,sysRole.getTenantId());
//			menuIds.forEach(e->{
//				sysRoleMenu.setMenuId(e);
//				sysRoleMenuDao.insert(sysRoleMenu);
//			});
		}
	}

}