package com.keao.edu.thirdparty.adapay;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huifu.adapay.core.exception.BaseAdaPayException;
import org.springframework.util.DigestUtils;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

public class Payment {
    /**
     * 执行一个支付交易
     *
     * @return 创建的支付对象
     * @throws Exception 异常
     */
    public static Map<String, Object> executePayment(Map<String, Object> paymentParams) throws Exception {
        paymentParams.put("app_id", ConfigInit.appId);

        //调用sdk方法，创建支付，得到支付对象
        Map<String, Object> payment = new HashMap<>();
        payment = com.huifu.adapay.model.Payment.create(paymentParams);

        String errorCode = (String) payment.get("error_code");
        if (null != errorCode) {
            String errorMsg = (String) payment.get("error_msg");
            throw new BaseAdaPayException(errorMsg);
        }

        return payment;
    }

    /**
     * 关闭一个支付交易
     *
     * @param paymentId 要关闭的支付id
     * @return 关闭的支付对象
     * @throws Exception 异常
     */
    public static Map<String, Object> closePayment(String paymentId) throws Exception {
        System.out.println("=======close payment begin=======");
        //关闭支付对象的参数，全部参数请参考 https://docs.adapay.tech/api/04-trade.html#id11
        //调用sdk方法，关闭支付，得到支付对象
        Map<String, Object> payment = new HashMap<>();
        Map<String, Object> paymentParams = new HashMap<>(10);
        paymentParams.put("payment_id", paymentId);
        try {
            paymentParams.put("payment_id", paymentId);
            paymentParams.put("reason", "reason");
            paymentParams.put("expend", "expend");
            paymentParams.put("notify_url", "notify_url");
            System.out.println("关单请求参数：" + JSON.toJSONString(paymentId));
            payment = com.huifu.adapay.model.Payment.close(paymentParams);
        } catch (BaseAdaPayException e) {
            e.printStackTrace();
        }


        String error_code = (String) payment.get("error_code");
        if (null != error_code) {
            System.out.println("关单返回参数：" + JSON.toJSONString(payment));
            String error_msg = (String) payment.get("error_msg");
            System.out.println("error_code:" + error_code + "............." + error_msg);
        }


        return payment;
    }

    /**
     * 查询一个支付交易
     *
     * @param paymentId 要查询的支付id
     * @return 查询的支付对象
     * @throws Exception 异常
     */
    public static Map<String, Object> queryPayment(String paymentId) throws Exception {
        System.out.println("=======query payment begin=======");
        //查询支付对象的参数，全部参数请参考 https://docs.adapay.tech/api/04-trade.html#id7
        //调用sdk方法，查询支付交易，得到支付对象
        Map<String, Object> payment = new HashMap<>();
        try {
            System.out.println("支付查询请求参数：" + JSON.toJSONString(paymentId));
            payment = com.huifu.adapay.model.Payment.query(paymentId);
        } catch (BaseAdaPayException e) {
            e.printStackTrace();
        }

        System.out.println("支付查询返回参数：" + JSON.toJSONString(payment));

        String error_code = (String) payment.get("error_code");
        if (null == error_code) {
            String error_msg = (String) payment.get("error_msg");
            System.out.println("error_code:" + error_code + "............." + error_msg);
        }
        System.out.println("=======query payment end=======");
        return payment;
    }


    public static Map<String, Object> queryList(Integer pageIndex, Long createdGte, Long createdLte) throws Exception {
        Map<String, Object> paymentParams = new HashMap<>();
        paymentParams.put("app_id", ConfigInit.appId);
        paymentParams.put("page_index", pageIndex);
        paymentParams.put("page_size", "20");
        paymentParams.put("created_gte", createdGte);
        paymentParams.put("created_lte", createdLte);
        Map<String, Object> payment = com.huifu.adapay.model.Payment.queryList(paymentParams);
        String error_code = (String) payment.get("error_code");
        if (null != error_code) {
            String errorMsg = (String) payment.get("error_msg");
            throw new BaseAdaPayException(errorMsg);
        }
        return payment;
    }

    public static Map<String, Object> getPayMap(BigDecimal amount, String orderNo,String notifyUrl,String returnUrl, String orderSubject, String orderBody) throws Exception {
        Map<String, Object> paymentParams = new LinkedHashMap<>();
        paymentParams.put("appId", ConfigInit.appId);
        paymentParams.put("amount", amount.setScale(2, BigDecimal.ROUND_HALF_UP));
        paymentParams.put("orderNo", orderNo);
//        paymentParams.put("notifyUrl", notifyUrl);
//        paymentParams.put("returnUrl", returnUrl);
        paymentParams.put("orderSubject", orderSubject);
        paymentParams.put("orderBody", orderBody);
        paymentParams.put("wxAppId", ConfigInit.wxAppId);

        String originalStr = JSONObject.toJSONString(paymentParams);
        String sign = DigestUtils.md5DigestAsHex(originalStr.getBytes());

        paymentParams.put("sign", sign);
        paymentParams.remove("appId");
        return paymentParams;
    }
}
