package com.keao.edu.user.controller;

import com.keao.edu.common.controller.BaseController;
import com.keao.edu.common.entity.HttpResponseResult;
import com.keao.edu.user.dto.ExamCertificationDto;
import com.keao.edu.user.dto.NeedCheckingDetailDto;
import com.keao.edu.user.service.ExamCertificationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
@RequestMapping("examCertification")
@Api(tags = "准考证服务")
public class ExamCertificationController extends BaseController {

    @Autowired
    private ExamCertificationService examCertificationService;

    @ApiOperation("后台获取学员准考证详情")
    @GetMapping(value = "findDetailByStudentId")
    @PreAuthorize("@pcs.hasPermissions('examCertification/findDetailByStudentId')")
    public HttpResponseResult<ExamCertificationDto> findDetailByStudentId(Long examRegistrationId) {
        return succeed(examCertificationService.findDetailByStudentId(examRegistrationId));
    }

    @ApiOperation("学生端获取学员准考证列表")
    @GetMapping(value = "queryCertificationPage")
    @PreAuthorize("@pcs.hasPermissions('examCertification/queryCertificationPage')")
    public HttpResponseResult<List<ExamCertificationDto>> queryCertification(Long examRegistrationId) {
        return succeed(examCertificationService.queryCertificationPage(examRegistrationId));
    }

    @ApiOperation("学生端待考详情")
    @GetMapping(value = "needCheckingDetail")
    @PreAuthorize("@pcs.hasPermissions('examCertification/needCheckingDetail')")
    public HttpResponseResult<NeedCheckingDetailDto> needCheckingDetail(Long examRegistrationId) {
        return succeed(examCertificationService.needCheckingDetail(examRegistrationId));
    }
}
