package com.keao.edu.user.controller;

import com.keao.edu.common.controller.BaseController;
import com.keao.edu.common.entity.HttpResponseResult;
import com.keao.edu.common.page.PageInfo;
import com.keao.edu.common.tenant.TenantContextHolder;
import com.keao.edu.user.entity.ExamSong;
import com.keao.edu.user.entity.Teacher;
import com.keao.edu.user.page.ExamSongQueryInfo;
import com.keao.edu.user.service.ExamSongService;
import com.keao.edu.util.date.DateUtil;
import com.keao.edu.util.excel.POIUtil;
import com.keao.edu.util.iniFile.IniFileEntity;
import com.keao.edu.util.iniFile.IniFileUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.util.ResourceUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @Author Joburgess
 * @Date 2020.06.16
 */
@RestController
@RequestMapping("examSong")
@Api(tags = "曲库服务")
public class ExamSongController extends BaseController {

    @Autowired
    private ExamSongService examSongService;

    @ApiOperation("分页查询")
    @GetMapping(value = "/list")
    @PreAuthorize("@pcs.hasPermissions('examSong/list')")
    public HttpResponseResult<PageInfo<ExamSong>> getList(ExamSongQueryInfo queryInfo) {
        return succeed(examSongService.queryPage(queryInfo));
    }

    @ApiOperation("学生端分页查询考级曲库")
    @GetMapping(value = "/queryPage")
    @PreAuthorize("@pcs.hasPermissions('examSong/queryPage')")
    public HttpResponseResult<PageInfo<ExamSong>> queryPage(ExamSongQueryInfo queryInfo) {
        return succeed(examSongService.querySongPage(queryInfo));
    }

    @ApiOperation("查询曲库详情")
    @ApiImplicitParam(name = "id", value = "机构ID", required = true, dataType = "Integer", paramType = "path")
    @GetMapping(value = "/query")
    @PreAuthorize("@pcs.hasPermissions('examSong/query')")
    public HttpResponseResult<ExamSong> query(Integer id) {
        return succeed(examSongService.get(id));
    }

    @ApiOperation("新增曲库")
    @PostMapping(value = "/add")
    @PreAuthorize("@pcs.hasPermissions('examSong/add')")
    public HttpResponseResult add(@RequestBody ExamSong examSong) {
        examSong.setTenantId(TenantContextHolder.getTenantId());
        examSongService.insert(examSong);
        return succeed();
    }

    @ApiOperation("更新曲库")
    @PostMapping(value = "/update")
    @PreAuthorize("@pcs.hasPermissions('examSong/update')")
    public HttpResponseResult update(@RequestBody ExamSong examSong) {
        examSong.setUpdateTime(new Date());
        examSongService.update(examSong);
        return succeed();
    }

    @ApiOperation("删除曲库")
    @PostMapping(value = "/del")
    @PreAuthorize("@pcs.hasPermissions('examSong/del')")
    public HttpResponseResult add(Integer id) {
        return succeed(examSongService.delete(id));
    }

    @ApiOperation("根据专业获取曲目")
    @GetMapping(value = "/getWithSubject")
    @PreAuthorize("@pcs.hasPermissions('examSong/getWithSubject')")
    public HttpResponseResult<List<ExamSong>> getWithSubject(Integer subjectId) {
        return succeed(examSongService.getWithSubject(subjectId));
    }

    @ApiOperation(value = "导入曲目")
    @PostMapping(value = "importExamSong")
    @PreAuthorize("@pcs.hasPermissions('examSong/importExamSong')")
    public HttpResponseResult<List<ExamSong>> importExamSong(@RequestParam("file") MultipartFile file) throws Exception {
        String tenantId = TenantContextHolder.getTenantId();
        List<ExamSong> examSongs = examSongService.importExamSong(file, tenantId);
        return succeed(examSongs);
    }

    @ApiOperation(value = "获取导入模板")
    @GetMapping(value = "getImportTemplate")
    @PreAuthorize("@pcs.hasPermissions('examSong/getImportTemplate')")
    public void getImportTemplate(HttpServletResponse response) throws IOException {
        InputStream inputStream = new ClassPathResource("excelTemplate/examSong.xls").getInputStream();
        OutputStream outputStream = response.getOutputStream();
        try {
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attachment;filename=examSong-" + DateUtil.getDate(new Date()) + ".xls");
            response.flushBuffer();
            outputStream = response.getOutputStream();
            IOUtils.copy(inputStream, outputStream);
            outputStream.flush();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }


}
