package com.keao.edu.user.controller;


import com.keao.edu.auth.api.client.SysUserFeignService;
import com.keao.edu.auth.api.entity.SysUser;
import com.keao.edu.common.controller.BaseController;
import com.keao.edu.common.entity.HttpResponseResult;
import com.keao.edu.common.page.PageInfo;
import com.keao.edu.common.tenant.TenantContextHolder;
import com.keao.edu.user.entity.Organization;
import com.keao.edu.user.page.OrganizationQueryInfo;
import com.keao.edu.user.service.OrganizationService;
import com.keao.edu.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.List;

/**
 * 合作单位服务
 */
@RestController
@Api(tags = "合作单位服务")
@RequestMapping(value = "organization")
public class OrganizationController extends BaseController {

    @Autowired
    private OrganizationService organizationService;
    @Autowired
    private SysUserFeignService sysUserFeignService;

    @ApiOperation(value = "合作单位列表")
    @GetMapping(value = "list")
    @PreAuthorize("@pcs.hasPermissions('organization/list')")
    public HttpResponseResult<PageInfo<Organization>> configList(OrganizationQueryInfo queryInfo) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        queryInfo.setParentId(sysUser.getId());
        if (queryInfo.getOrganId() != null) {
            queryInfo.setOrganIds(organizationService.getChildOrganIds(queryInfo.getOrganId(), false));
        }
        return succeed(organizationService.queryPage(queryInfo));
    }

    @ApiOperation(value = "修改合作单位")
    @PostMapping(value = "update")
    @PreAuthorize("@pcs.hasPermissions('organization/update')")
    public HttpResponseResult updateOrganization(@RequestBody Organization organization) {
        organizationService.updateOrganization(organization);
        return succeed();
    }

    @ApiOperation(value = "新增合作单位")
    @PostMapping(value = "add")
    @PreAuthorize("@pcs.hasPermissions('organization/add')")
    public HttpResponseResult addConfig(@RequestBody Organization organization) {
        organization.setTenantId(TenantContextHolder.getTenantId());
        organizationService.add(organization);
        return succeed();
    }

    @ApiOperation(value = "查询合作单位")
    @GetMapping(value = "get")
    @PreAuthorize("@pcs.hasPermissions('organization/get')")
    public HttpResponseResult<Organization> getConfig(Integer id) {
        return succeed(organizationService.getOrganization(id));
    }

    @ApiOperation(value = "删除合作单位")
    @PostMapping(value = "del")
    @PreAuthorize("@pcs.hasPermissions('organization/del')")
    public HttpResponseResult del(Integer id) {
        organizationService.del(id);
        return succeed();
    }

    @ApiOperation(value = "启用/停用合作单位")
    @PostMapping(value = "lock")
    @PreAuthorize("@pcs.hasPermissions('organization/lock')")
    public HttpResponseResult lock(Integer id, Integer lockFlag) {
        organizationService.lock(id, lockFlag);
        return succeed();
    }

    @ApiOperation(value = "获取当前分部及其所有子合作单位")
    @GetMapping(value = "getChildOrgans")
    @PreAuthorize("@pcs.hasPermissions('organization/getChildOrgans')")
    public HttpResponseResult<List<Organization>> getChildOrgans(Integer includeSelf) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        Organization organization = organizationService.findByUserId(sysUser.getId());
        return succeed(organizationService.getChildOrgans(organization.getId(), includeSelf == 1));
    }

    @ApiOperation(value = "导入合作单位")
    @PostMapping(value = "importOrgan")
    @PreAuthorize("@pcs.hasPermissions('organization/importOrgan')")
    public HttpResponseResult<List<Organization>> importOrgan(@RequestParam("file") MultipartFile file) throws Exception {
        String tenantId = TenantContextHolder.getTenantId();
        List<Organization> organizations = organizationService.importOrgan(file, tenantId);
        return succeed(organizations);
    }

    @ApiOperation(value = "获取导入模板")
    @GetMapping(value = "getImportTemplate")
    @PreAuthorize("@pcs.hasPermissions('organization/getImportTemplate')")
    public void getImportTemplate(HttpServletResponse response) throws IOException {
        InputStream inputStream = new ClassPathResource("excelTemplate/organization.xls").getInputStream();
        OutputStream outputStream = response.getOutputStream();
        try {
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition", "attachment;filename=organization-" + DateUtil.getDate(new Date()) + ".xls");
            response.flushBuffer();
            outputStream = response.getOutputStream();
            IOUtils.copy(inputStream, outputStream);
            outputStream.flush();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
    }

}
